@@ -1,8 +1,12 @@
 import os
+import os.path
 import sys
 
 import pytest
 
+# needed to get pretty assertion failures in unit tests:
+if hasattr(pytest, 'register_assert_rewrite'):
+    pytest.register_assert_rewrite('borg.testsuite')
 
 # This is a hack to fix path problems because "borg" (the package) is in the source root.
 # When importing the conftest an "import borg" can incorrectly import the borg from the
@@ -10,12 +14,10 @@ import pytest
 # The workaround is to remove entries pointing there from the path and check whether "borg"
 # is still importable. If it is not, then it has not been installed in the environment
 # and the entries are put back.
-#
-# TODO: After moving the package to src/: remove this.
 
 original_path = list(sys.path)
 for entry in original_path:
-    if entry == '' or entry.endswith('/borg'):
+    if entry == '' or entry == os.path.dirname(__file__):
         sys.path.remove(entry)
 
 try:
